from frame.frame_core.frames import *
from frame.frame_core.plugins import MathPlugin
from frame.frame_core.funcs import has_module
import os, subprocess, time, ast




class FramesComposer:
    '''
# FramesComposer
Fames archestrator. Based by 2 archs: dict (where indexes is any data) and array (where indexes is integrers).

## Args:
- {safemode}: bool - superglobal.safemode state.
- {arch}: str[dict/array (array is VERY experemental)] - arch base.
- {superglobal_name}: str - name of superglobal context.

## Functions:
- [fast_import] - import module to compose.
- [load_fame] - load frame to compose.
- [check_deps] - check dependencies.
- [get_frame] - get frame from compose.
- [sync] - sync data of 2 frames.
- [deploy] - run and deploy compose method.
- [_data] - compile composer to dict.
- [_load_data] - load compiled data to compose from dict.
- [save/load] - serealize methods.
- [from_file] (classmethod) - create compose from file.
- [test_exec] - checks and test run compose.
- [__call__] - return superglobal context.
- [__getitem__] - get frame by name.
- [__setitem__] - load frame by name, format `compose[name] = frame`.
- [__add__] - load frame with format like `compose + frame`.
- [context manager `with` support]
- [else systems]

## Example:
```
with FramesComposer(safemode=False) as fc:
    fc['test1'] = Frame(safemode=False)
    fc['test2'] = Frame(safemode=False)
    @fc['test2'].register()
    def test():
        return 'compleate'
    with fc['test2'] as f:
        f.Var('x', 10)
        f.Var('y', 50)
```
    '''
    def __init__(self, safemode: bool = False, arch: str = 'dict', superglobal_name: str = 'sgc'):
        framer = 'new' 
        self.__safemode = safemode
        save_while_exit = False
        save_args = ['ctx', 'pickle']
        self._superglobal_name = superglobal_name
        self.sgc = Frame(framer, self.__safemode, superglobal_name, save_while_exit, save_args) # superglobal context
        self._frames: list[Frame] | dict[str, Frame] = {} if arch == 'dict' else []
        self._arch = arch
        self._deps = []
        self.__temp_names = []
    def fast_import(self, modulename):
        'fast module import into compose'
        if has_module(modulename): self.sgc.Code(f'import {modulename}')
        else: self.sgc.Code(f'"!WARNING: MODULE CAN BE IS NOT INSTALLED!"\nimport {modulename}')
    def load_frame(self, 
                   index: str | int, 
                   frame: Frame, 
                   add_to_deps: bool = True) -> 'FramesComposer':
        '''
#### Create frame in composer.
#### Args: 
- {index}: int | str - new frame index.
- {frame}: Frame - frame object to append into composer.
- {add_to_deps}: bool - add frame name to dependencies if true, else pass.
        '''
        self.__temp_names.append(index)
        self._deps.append(index) if add_to_deps else None
        if self._arch == 'dict': self._frames[index] = frame
        else: 
            pre = self._frames[:index-1]
            after = self._frames[index:]
            _frames = pre + [frame] + after
            self._frames = _frames
        return self
    def check_deps(self):
        '''
Check dependencies of composer.
        '''
        for i in self._deps: 
            if i not in self.__temp_names:
                raise FrameComposeError(i, 'FRAME NOT FOUND', f'Dependency is not found: [{i}].')
    def get_frame(self, index: str | int) -> Frame:
        '''
Get frame by {index}.
        '''
        try: return self._frames[int(index) if self._arch == 'array' else index]
        except (IndexError, KeyError) as e: raise FrameComposeError(index, 'GetFrameError', e)
    def sync(self, name_1: str, name_2: str, algoritm: int = 1) -> 'FramesComposer':
        '''
Sync 2 frames with algoritm.

Algoritms:
    - 1: sync variables
    - 2: algoritm 1 + sync code
        '''
        'algoritms: 1 - async code, 2 - sync code'
        if name_1 not in ('$', 'sgc'):
            f1 = self._frames[name_1]  
        else: f1 =  self.sgc
        if name_2 not in ('$', 'sgc'):
            f2 = self._frames[name_2]
        else: f2 = self.sgc
        new_dict1 = {}
        for i in f1.framer._aliases: 
            new_dict1[i] = f1.framer._aliases[i]
        for i in f2.framer._aliases: 
            new_dict1[i] = f2.framer._aliases[i]
        f1.framer._aliases = new_dict1
        f2.framer._aliases = new_dict1
        new_dict2 = {}
        for i in f1.framer._vars: 
            new_dict2[i] = f1.framer._vars[i]
        for i in f2.framer._vars: 
            new_dict2[i] = f2.framer._vars[i]
        f1.framer._vars = new_dict2
        f2.framer._vars = new_dict2
        if algoritm == 2:
            f1c = f1.framer._code
            f1.framer._code = f1c + f2.framer._code
            f2.framer._code = f2.framer._code + f1c
        if name_1 not in ('$', 'sgc'): self._frames[name_1] = f1
        else: self.sgc = f1
        if name_2 not in ('$', 'sgc'): self._frames[name_2] = f2
        else: self.sgc = f2
        return self
    def superglobal(self) -> Frame: 'get superglobal context of class'; return self.sgc
    def deploy(
        self,
        name: str,
        fcomp_filename: str,
        fcomp_format: str = 'json',
        main_code: str = '',
        runfile_dir: str = os.getcwd(),
        metadata: str = '',
        version: str = None,
        author: str = None,
        dependencies: list = None,
        runing: bool = False
    ):
        '''
# Deploy compose
Method to compile and deploy your compose to file and run (optional).

## Args:
- {name}: str - filename to deployfile.
- {fcomp_filename}: str - fcomp iso dilename.
- {fcomp_format}: str - fcomp format.
- {main_code}: str - deployfile code.
- {runfile_dir}: str - main dir for deployfile.
- {metadata}: str - metadata for deployfile.
    - {version}: str - metadata
    - {author}: str - metadata
- {dependencies}: str - deps for deploy
- {running}: bool - running deploy file if true, else pass

## Example: 
```
with FramesComposer.from_file(filepath, format) as fc: fc.deploy('test.py', 'fc.json', runing=True)
```
This is simple deploy of file.
        '''
        runfile_dir = runfile_dir.replace('$', os.getcwd()) if runfile_dir.startswith('$') else runfile_dir
        self.check_deps()
        if self.__safemode:
            raise FrameComposeError(f'Superglobal Context [{self._superglobal_name}]', 'EXEC ERROR',
            'FramesCompose deploy is imposible in safemode.')
        _cond = dependencies not in (None, [])
        imports = ['from frame import *'] 
        if _cond:
            for i in dependencies: imports.append(f'import {i}')
        def metadata_compiler(mtdt):
            mtdt = f'deploy_time = {time.time()}{"\n" if mtdt else ""}' + mtdt
            counter = 1
            max_lines = 3
            new_mtd = ''
            for i in mtdt.split('\n'):
                new_mtd +=  f'    {counter}{" " * (max_lines-len(str(counter)))}| ' + str(i) + '\n'
                counter += 1
            return new_mtd
        metadata = metadata_compiler(metadata)
        code = f'''
# start
{'\n'.join(imports)}
"""

==========================================
Deploy file [{name}] for fcomp by [{author}]
Version: {version}
Metadata:
{metadata}
==========================================

"""

# fcirf - frames composotion iso runtime file
_fcirf_version_ = {repr(version)}
_fcirf_name_ = {repr(name)}
_fcirf_author_ = {repr(author)}
_fcirf_dir_ = {repr(runfile_dir)}
_fcirf_deps_ = {dependencies}
_fcirf_fcomp_info_ = {[fcomp_filename, fcomp_format]}

fcomp = FramesComposer.from_file(_fcirf_fcomp_info_[0], _fcirf_fcomp_info_[1])
sgc = fcomp.superglobal

{main_code}
        '''
        pth = os.getcwd()
        os.chdir(runfile_dir)
        with open(name, 'w') as file:
            file.write(code)
        if runing: 
            command = f'python3 {name}'
            os.system(command=command)
        os.chdir(pth)
        return self
    def _data(self):
        '''Compile composer to dictionary data.'''
        data = {'_frames': {}}
        data['_superglobal_name'] = self._superglobal_name
        data['_arch'] = self._arch
        data['_deps'] = self._deps
        data['__safemode'] = self.__safemode
        data['sgc'] = self.sgc.data
        for i in self._frames.keys(): 
            data['_frames'][i] = self._frames[i].data
        return data
    def _load_data(self, data: dict):
        """Load data into existing FramesComposer instance"""
        self._superglobal_name = data['_superglobal_name']
        self._arch = data['_arch']
        self._deps = data['_deps']
        self.__safemode = data['__safemode']
        # Load superglobal context
        self.sgc._load_data(data['sgc'])
        # Clear current frames
        self._frames = {} if self._arch == 'dict' else []
        # Load all frames
        if isinstance(data['_frames'], dict): fr = data['_frames'].items()
        elif isinstance(data['_frames'], list): fr = data['_frames']
        else:
            raise FrameComposeError('', 'UNKNOWN TYPE TO LOAD', 'Unknown arch for load fcomp file.')
        for frame_name, frame_data in fr:
            frame = Frame()
            frame._load_data(frame_data)
            self.load_frame(frame_name, frame)
    def save(self, filename, format: str = 'json'):
        '''
        ## Saving FramesComposer to file.
        ### Args:
        - {filename}: str - file name
        - {format}: str - saving format ('pickle' or 'json')
        ''' 
        data = self._data()
        try:
            if format == 'pickle':
                with open(filename, 'wb') as f: 
                    pickle.dump(data, f)
            elif format == 'json':
                with open(filename, 'w', encoding='utf-8') as f: 
                    json.dump(data, f, indent=2, ensure_ascii=False)
            else: 
                raise FrameApiError(f"Unsupported format: {format}")
            return self
        except Exception as e:
            raise FrameApiError(f"Save failed: {e}")
    
    def load(self, filename: str, format: str = 'json') -> 'FramesComposer':
        '''
        ## Loading FramesComposer from file.
        ### Args:
        - {filename}: str - file name  
        - {format}: str - loading format ('pickle' or 'json')
        ### Returns:
        - FramesComposer: self for method chaining
        '''
        try:
            if format == 'pickle':
                with open(filename, 'rb') as f:
                    data = pickle.load(f)
            elif format == 'json':
                with open(filename, 'r', encoding='utf-8') as f:
                    data = json.load(f)
            else:
                raise FrameApiError(f"Unsupported format: {format}")
            
            self._load_data(data)
            return self
            
        except FileNotFoundError:
            raise FrameApiError(f"File not found: {filename}")
        except Exception as e:
            raise FrameApiError(f"Load failed: {e}")
    @classmethod
    def from_file(cls, filename: str, format: str = 'json') -> 'FramesComposer':
        '''
        ## Create FramesComposer from file (class method)
        ### Args:
        - {filename}: str - file name
        - {format}: str - loading format ('pickle' or 'json')
        ### Returns:
        - FramesComposer: new instance loaded from file
        '''
        composer = cls()
        composer.load(filename, format)
        return composer
    def test_exec(self) -> 'FramesComposer':
        self.check_deps()
        if self.__safemode:
            raise FrameComposeError(f'Superglobal Context [{self._superglobal_name}]', 'EXEC ERROR',
            'Execution is imposible in safemode.')
        self.sgc.Exec()
        return self
    def __enter__(self) -> 'FramesComposer': return self
    def __exit__(self, *args, **kwargs): pass
    def __call__(self, *args, **kwds) -> Frame: return self.superglobal()
    def __add__(self, other: Frame) -> None: 
        if isinstance(other, Frame): self.load_frame(len(self._frames) if self._arch == 'array' else other._name, other)
        else: raise FrameComposeError('', 'NotSuuportableObject', f"Inncorect attemp to add {type(other)} object to frames.")
    def __getitem__(self, index) -> Frame:
        try: 
            return self.sgc if index == self._superglobal_name or str(index).strip() == '$' else self._frames[index]
        except (IndexError, KeyError) as e: raise FrameComposeError(index, 'GetFrameError', f'Unknown key: {e}')
        except Exception as e: raise FrameComposeError(index, 'GetItemError', e)
    def __setitem__(self, index, value):
        try: self.load_frame(index, value)
        except (IndexError, KeyError) as e: raise FrameComposeError(index, 'GetFrameError', f'Unknown key: {e}')
        except Exception as e: raise FrameComposeError(index, 'SetItemError', e)
    def __eq__(self, value) -> bool:
        if isinstance(value, FramesComposer): 
            cond1 = value.__safemode == self.__safemode and value._arch == self._arch
            cond2 = value.sgc._name == self.sgc._name  and value._superglobal_name == self._superglobal_name
            return cond1 and cond2
        else: return False
    def __format__(self, format_spec: str) -> str:
        if format_spec == '.all': return str(self._frames)
        elif format_spec.startswith('.get>'):
            index = format_spec[5:]
            list = {}
            counter = 0
            for i in self._frames: 
                list[str(counter)] = self._frames[i]
                counter += 1
            try: return str(list[str(index)])
            except (KeyError) as e: 
                raise FrameComposeError(f'index<{index}>', 'GetItemError', f'Unknown key: {e}')
            except Exception as e: raise FrameComposeError(index, 'fStringError', e)
        elif format_spec.startswith('.getname>'):
            index = format_spec[9:]
            list = {}
            counter = 0
            for i in self._frames: 
                list[str(counter)] = i
                counter += 1
            try: return list[str(index)]
            except (KeyError) as e: 
                raise FrameComposeError(f'index<{index}>', 'GetItemError', f'Unknown key: {e}')
            except Exception as e: raise FrameComposeError(index, 'fStringError', e)
        elif format_spec.startswith('.safemode'):
            return str(self.__safemode)
        elif format_spec.startswith('.hash'):
            return str(self.__hash__())
        elif format_spec.startswith(('.sgc', '.superglobal', '.sgcname')): 
            return self.sgc._name
        elif format_spec.startswith(('.arch', '.a')): 
            return self._arch
        raise ValueError('Unknown format option.')
    def __hash__(self) -> int:
        arch = str_to_int(self._arch)
        frames = str_to_int(self._frames)
        safemode = str_to_int(self.__safemode)
        superglobal = str_to_int(self._superglobal_name)
        return arch+frames+safemode+superglobal
    def __int__(self):
        return self.__hash__()



if __name__ == '__main__':
    filename = 'fc'
    format = 'json'
    filepath = f'{filename}.{format}'
    with FramesComposer(safemode=False) as fc:
        fc['test1'] = Frame(safemode=False)
        fc['test2'] = Frame(safemode=False)
        @fc['test2'].register()
        def test():
            return 'compleate'
        with fc['test2'] as f:
            f.Var('x', 10)
            f.Var('y', 50)
            SystemOP.match('x > y', 'print("x bigger")', 'print("y bigger")')
            f.Var('test', Get('x') * Get('y')) 
            @f.register()
            def test(): 
                print('testing')
            @f.register()
            class Test():
                hello = 'World'
                pass
        @fc['test1'].register()
        def test():
            return 'compleate'
        mfc = MathPlugin(fc['test1']).include()
        mfc.discriminant(10, 20, 30)
        mfc.discriminant(20, 20, 80)
        fc.sync('test1', '$')
        fc.sync('test2', '$')
        fc.save(filepath, format)
    with FramesComposer.from_file(filepath, format) as fc:
        fc.test_exec()
        fc.deploy('test.py', 'fc.json', runing=True)




import unittest
import time
import sys
import os

sys.path.append(os.path.dirname(os.path.dirname(__file__)))

from frame import Frame, FramesComposer, fVar, fExec, FastGet

class TestPerformance(unittest.TestCase):
    
    def test_execution_speed(self):
        """Тест скорости выполнения"""
        start_time = time.time()
        
        with Frame(safemode=False) as f:
            # Создаем 50 переменных (уменьшил для скорости)
            for i in range(50):
                fVar(f'var_{i}', i, framer=f())
            
            # Выполняем простые операции
            fVar('sum', 'sum([var_0, var_49])', with_eval=True, framer=f())
            result = FastGet(f, 'sum')
        
        execution_time = time.time() - start_time
        self.assertEqual(result, 49)  # 0 + 49 = 49
        self.assertLess(execution_time, 2.0, "Выполнение заняло слишком много времени")
    
    def test_memory_efficiency(self):
        """Тест эффективности использования памяти"""
        # Создаем множество фреймов
        frames = []
        for i in range(20):  # Уменьшил количество для стабильности
            frame = Frame()
            for j in range(10):
                frame.Var(f'data_{j}', f'value_{i}_{j}')
            frames.append(frame)
        
        # Если не упало по памяти - тест пройден
        self.assertEqual(len(frames), 20)

if __name__ == '__main__':
    unittest.main()


import secrets as s, abc as a
from .bip39_worldlist import wordlist
rnd = s.SystemRandom()


def gencode_int(length: int = 10):
    result = []
    for _ in range(length): result.append(rnd.randrange(0, 9))
    res = []
    for _ in range(length): res.append(rnd.choice(result))
    result1 = ''
    for i in res: result1 += str(i)
    return result1

def gencode_int_sections(sections: int = 2, one_section_length: int = 4, separator: str = '-'):
    sections_tmp = []
    for i in range(sections): sections_tmp += [gencode_int(one_section_length)]
    result2 = ''
    for i in sections_tmp:
        for li in i: result2 += str(li)
        result2 += separator
    return result2[:-1]

def gencode_str(length: int = 10):
    alphabet = list('1234567890' +
                    'QwErTyUiOpAsDfGhJkLzXcVbNmqWeRtYuIoPaSdFgHjKlZxCvBnM')
    result = []
    for _ in range(length): result.append(rnd.choice(alphabet))
    return ''.join(result)

def gencode_str_sections(sections: int = 4, one_section_length: int = 5, separator: str = '-'):
    sections_tmp = []
    for i in range(sections): sections_tmp += [gencode_str(one_section_length)]
    return separator.join(sections_tmp)

def genpass(length: int = 20):
    if length < 8: raise ValueError(f"Can't generate password with length < 8. Current length: [{length}].")
    res = []
    for _ in range(round(length/2)): res += gencode_str(2)
    return ''.join(res)

def gen_mnemonic_phrase(word_count=12):
    return ' '.join(s.choice(wordlist) for _ in range(word_count))


class ClassApi(a.ABC):
    def __init__(self):
        super().__init__()
    
    @staticmethod
    def gen_type(type: str = 'int', length: int = 10):
        'type: int/str'
        if type == 'int': return gencode_int(length)
        else: return gencode_str(length)
    
    @staticmethod
    def gen_type_sections(type: str = 'int', 
                          sections: int = 4, 
                          one_section_length: int = 5, 
                          separator: str = '-'):
        'type: int/str'
        if type == 'int': return gencode_int_sections(sections, one_section_length, separator)
        else: return gencode_str_sections(sections, one_section_length, separator)
    
    @staticmethod
    def gen_password(length: int = 8): return genpass(length)

    @staticmethod
    def gen_mnemonic(length: int = 10): return gen_mnemonic_phrase(length)


import hashlib, abc, functools, sys
#sys.set_int_max_str_digits(50000) 


def str_to_int(string: str, summing: bool = True) -> list | int:
    string = str(string)
    alphabet = list('1234567890 ' +
                    'QWERTYUIOPASDFGHJKLZXCVBNMqwertyuiopasdfghjklzxcvbnm' +
                    'ЙЦУКЕНГШЩЗХЪФЫВАПРОЛДЖЭЁЯЧСМИТЬБЮйцукенгшщзхъфывапролджэёячсмитьбю' +
                    '!@#$%^&*()_+¡™£¢∞§¶•ªº–≠[]{}\\|/?.,<>\'":;±§`~\n\t')
    res = []
    for i in list(string):
        if i in alphabet: res.append(alphabet.index(i)+1)
        else: 
            alphabet.append(i)
            res.append(alphabet.index(i)+1)
    return sum(res) if summing else res

def int_to_str(integrear: str, to_join: bool = True) -> list:
    integrear = str(integrear)
    alphabet = list('1234567890-' +
                    'QwErTyUiOpAsDfGhJkLzXcVbNmqWeRtYuIoPaSdFgHjKlZxCvBnM')
    res = []
    local_alphabet = alphabet.copy()
    kf = str_to_int(integrear)
    for i in list(integrear):
        try:
            i = int(i) % (kf) + int(i)
            res.append(local_alphabet[i])
            local_alphabet = local_alphabet[:i] + local_alphabet[i+1:]
            if len(local_alphabet) < 10: local_alphabet=alphabet.copy()
        except: 
            local_alphabet=alphabet.copy()
            i = int(i) % (kf) + int(i)
            res.append(local_alphabet[i])
            local_alphabet = local_alphabet[:i] + local_alphabet[i+1:]
            if len(local_alphabet) < 10: local_alphabet=alphabet.copy()
    return ''.join(res) if to_join else res


def sha512_sha256_hash(inp):
    inp = str(inp).encode()
    h1 = hashlib.sha3_512(inp)
    h2 = hashlib.sha3_256(h1.hexdigest().encode())
    return h2.hexdigest()


def our(inp):
    done_int = str_to_int(inp)
    first_digit = int(list(str(done_int))[0])
    last_digit = int(list(str(done_int))[-1])
    res = []
    for i in range(10+first_digit):
        done_int = int(''.join(reversed(list(str((done_int << 3 + i) >> 4))))) - i + last_digit * first_digit
        res += [int_to_str(done_int+i*last_digit)]
    return str_to_int(str(res))
    
def to_id(inp):
    integrear: int = str_to_int(inp)
    id_our = our(f'{inp}{integrear}')
    return f'{id_our}{integrear}{id(inp)}'



class ClassApi(abc.ABC):
    def __init__(self):
        super().__init__()
    
    @staticmethod
    def sha512_sha256_hash(inp): return sha512_sha256_hash(inp)

    @staticmethod
    def hash_pattern(inp): return our(inp)

    @staticmethod
    class system:
        def str_to_int(string: str, summing: bool = True): return str_to_int(string, summing)
        def int_to(integrear: int, join: bool = True): return str_to_int(integrear, join)




from cryptography.fernet import Fernet
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC
import base64, abc, hashlib, os, time, threading as th, secrets




class WarningType:
    def __init__(self, data):
        self.raw = data
        self.data = f'WARNING <[{data}]>'
    def __new__(self, data, to_object:bool=False):
        self.raw = data
        self.data = f'WARNING <[{data}]>'
        return self if to_object else self.data
    def __repr__(self):
        return f"'{self.data}'"
    def __eq__(self, value): return self.raw == value
    def __str__(self): return self.data
    def get(self): return self.data




class SafeData:
    """
    Data class for important constants in your program.
    Cryptography defender for data out of the box.
    Isn't works with custom classes and dtypes.
    Args:
        data: Any                    - data without encrypt
        key: Any                     - key for encrypting
        salt_for_key: bytes          - salt for encrypting
        time_to_live: int | 'inf'    - if you input int - ttl=time_to_live,
                                       if you input 'inf' - ttl=None
    User methods:
        'of'     - full encrypt for data
        'get'    - get data with decrypt
    """
    def __init__(self, data, key, 
                 salt_for_key: bytes=os.urandom(32), 
                 time_to_live :int|str='inf', 
                 threadlock_for_wait :bool=True):
        self._rnd_delay()
        key = self._hash(key).encode()
        kfd = PBKDF2HMAC(
    algorithm=hashes.SHA256(),
    length=32,
    salt=salt_for_key,
    iterations=100000)
        self.time = time_to_live
        self.threadlock = threadlock_for_wait
        self.t = th.Thread(target=self._wait)
        self.t.start()
        self.working = True
        self.key = base64.urlsafe_b64encode(kfd.derive(key))
        self.encrypted_data = self._encrypt(data, self.key)
        self.decrypted = False
        self.raw = self._hash(data)
        self.counter = 0
        self.logs = []
    
    def _rnd_delay(self):
        random = secrets.SystemRandom()
        delay = (random.randrange(100, 900) * 0.00003) + (random.randrange(100, 800) * 0.00001)
        time.sleep(delay)
    
    def _wait(self):
        try:
            if isinstance(self.time, (int, float)) and self.time > 0:
                time.sleep(self.time)
            elif self.time != 'inf':
                raise ValueError(f"Invalid TTL: {self.time}")
            if self.threadlock:
                with th.Lock():
                    self.working = False
            else:
                self.working = False
        except:
            if self.threadlock:
                with th.Lock():self.working = True
            else:self.working = True

    def _log(self, log):
        time_str = str(time.time())
        tm = time_str + ' ' * (20 - len(time_str))
        lg = str(log) + ' ' * (35 - len(str(log)))
        self.logs.append(f'LOG <[ {tm} : {lg} ]>')
    
    def _encrypt(self, data, key):
        cipher = Fernet(key)
        enscrypted_data = cipher.encrypt(str(data).encode())
        del data
        return enscrypted_data
    
    def _dec(self):
        self._log('=== DECRYPTING ===')
        if not self.working:
            self._undec_enc('phrase'.encode())
            self._log('Decryption failed - is not working')
            return WarningType('NOT_FOUND').encode()
        else: 
            kfd = self.key
            cipher = Fernet(kfd)
            decrypted_data = cipher.decrypt(self.encrypted_data)
            self.decrypted = True
            self._log('Decryption done')
            return decrypted_data

    def _enc(self, data):
        self._log('=== ENCRYPTING ===')
        cipher = Fernet(self.key)
        enscrypted_data = cipher.encrypt(str(data).encode())
        self.decrypted = False
        return enscrypted_data
    
    def _undec_enc(self, a):
        self._log('Full encrypting')
        random = secrets.SystemRandom()
        kfd = PBKDF2HMAC(
            algorithm=hashes.SHA256(),
            length=32,
            salt=os.urandom(32),
            iterations=100000)
        self.working = False
        self.encrypted_data = self._encrypt(a, base64.urlsafe_b64encode(kfd.derive(str(random.randint(10000, 90000)).encode())))
        del kfd
    
    def _check(self) -> tuple[bool, str]:
        des = self._dec(); self._enc(des)
        h = self.raw == self._hash(des.decode())
        d = self.decrypted == False
        if not h:
            self._log('Hash Error')
        if not d:
            self._log('Decryption Error')
        if not self.working:
            self._log("Isn't working Error")
        res =  d and h and self.working
        return (res, des)
    
    def _hash(self, data):
        return hashlib.sha256(str(data).encode()).hexdigest()

    def __enter__(self):
        self._log('New context manager')
        return self

    def __exit__(self, *args):
        self.time = 0.00001
        self._wait()
        self._undec_enc('warning'.encode())
        del self
        del args
        return 0
    
    def __call__(self, *args, **kwds):
        return self.get(args[0] if args else True)
    
    def get(self, check: bool=True, eval: bool=True):
        self.counter += 1
        ch = self._check() if check else [True, self._dec()]
        self._log(f'=== GET ===')
        try:
            self._rnd_delay()
            if ch[0]: 
                self._log('Get working')
                try:
                    import ast
                    self._rnd_delay()
                    return ast.literal_eval(ch[1].decode()) if eval else ch[1].decode()
                except:
                    self._rnd_delay()
                    return ch[1].decode()
            else: 
                self._log('Get failed')
                if self.working:
                    self._rnd_delay()
                    return WarningType(ch[1].decode())
                else:
                    self._rnd_delay()
                    self._undec_enc(ch[1])
                    return WarningType('NOT_FOUND')
        except Exception as e:
            self._rnd_delay()
            print(f'Error in <MDData.get>: {e}')
            return self

    def off(self):
        self._log('FULL DATA ENCRYPT')
        self.working = False
        self._dec()
        return self
    
    def count(self):
        return self.counter
    
    def get_logs(self, to_string: bool=True, to_join: str='\n'):
        res = to_join.join(self.logs) if to_string else self.logs
        return res
    
    def __del__(self):
        self._undec_enc('DELETE')
        del self.encrypted_data, self.raw
        self.raw = 'DELETED'
        self.encrypted_data = b'0x0000'




class ClassApi(abc.ABC):
    def __init__(self):
        super().__init__()
    
    @staticmethod
    def SafeDataClass(): return SafeData



if __name__ == '__main__':
    with SafeData('test', 'test') as data:
        print(data.get())
        print(data.off())
        print(data.get())


'''
# Nets.hack Mocule

Modules - 
    gray - gray hacking.
    white - white hacking.

### !Attention! - The author is not responsible for the user's actions. Only for legal usage.
'''


from .gray import (ClassApi as GrayHackApi)
from .white import (ClassApi as WhiteHackApi)


class HackModuleApi:
    @property
    def white(): return WhiteHackApi()
    @property
    def gray(): return GrayHackApi()



'''
# Nets.Crypto Module

Modules - 
    - hashes - hash methods.
    - rand_system - randow generating system based on [secrets].
'''

from .hashes import (ClassApi as HashCryptoApi)
from .rand_system import (ClassApi as RandCryptoApi)
from .bip39_worldlist import (wordlist as bip39wl)


import unittest
import sys
import os
import tempfile

sys.path.append(os.path.dirname(os.path.dirname(__file__)))

from frame import FramesComposer, Frame, fVar, FastGet

class TestFramesComposer(unittest.TestCase):
    
    def test_composer_creation(self):
        """Тест создания композитора"""
        with FramesComposer() as fc:
            self.assertIsNotNone(fc.superglobal())
            self.assertEqual(fc._arch, 'dict')
    
    def test_frame_loading(self):
        """Тест загрузки фреймов в композитор"""
        with FramesComposer() as fc:
            frame1 = Frame(name='test1')
            frame2 = Frame(name='test2')
            
            fc.load_frame('processor', frame1)
            fc.load_frame('validator', frame2)
            
            self.assertIn('processor', fc._frames)
            self.assertIn('validator', fc._frames)
    
    def test_frame_synchronization(self):
        """Тест синхронизации фреймов"""
        with FramesComposer(safemode=False) as fc:
            fc['frame1'] = Frame(safemode=False)
            fc['frame2'] = Frame(safemode=False)
            
            fc['frame1'].Var('shared_data', 100)
            fc.sync('frame1', 'frame2')
            
            # После синхронизации frame2 должен видеть переменную
            self.assertEqual(fc['frame2'].Get('shared_data'), 100)
    
    def test_composer_serialization(self):
        """Тест сериализации композитора"""
        with tempfile.NamedTemporaryFile(suffix='.json', delete=False) as temp_file:
            filename = temp_file.name
        
        try:
            # Создаем и сохраняем композицию
            with FramesComposer(safemode=False) as fc:
                fc['main'] = Frame(safemode=False)
                fc['main'].Var('app_state', 'running')
                fc.save(filename, format='json')
            
            # Загружаем композицию
            loaded_fc = FramesComposer.from_file(filename, format='json')
            self.assertEqual(loaded_fc['main'].Get('app_state'), 'running')
            
        finally:
            os.unlink(filename)
    
    def test_composer_execution(self):
        """Тест выполнения композитора"""
        with FramesComposer(safemode=False) as fc:
            fc['math_frame'] = Frame(safemode=False)
            fc['math_frame'].Var('x', 10)
            fc['math_frame'].Var('y', 5)
            fc['math_frame'].Var('result', 'x * y', with_eval=True)
            
            # Выполняем всю композицию
            fc.test_exec()
            
            # Проверяем результат
            self.assertEqual(FastGet(fc['math_frame'], 'result'), 50)

if __name__ == '__main__':
    unittest.main()


class FramerError(Exception):
    def __init__(self, *args):
        super().__init__(*args)
class FrameApiError(Exception):
    def __init__(self, *args):
        super().__init__(*args)
class FramingError(Exception):
    def __init__(self, *args):
        super().__init__(*args)
class PluginIsNotWorkingError(Exception):
    def __init__(self, *args):
        super().__init__(*args)
class PluginError(Exception):
    def __init__(self, *args):
        super().__init__(*args)
class FrameExecutionError(Exception):
    def __init__(self, *args):
        super().__init__(*args)
class VariableTypeError(Exception):
    def __init__(self, *args):
        super().__init__(*args)
class FrameComposeError(Exception):
    def __init__(self, framer_name = '', exception = '', exception_text = ''):
        self.text = f'''\n
Error in [Framer{'_' + str(framer_name) if str(framer_name) else ''}]
=== {exception} ===
{exception_text}
{'=' * (len(exception) + 8)}
'''
        super().__init__(self.text)




from abc import ABC
from .frames import Frame, exec_and_return, PluginIsNotWorkingError, PluginError
import math, cmath, random, importlib





class PluginBase(ABC):
    '''
# Base for any plugins
System objects: 
- frame - inner frame.
- _has_frame - isinstance {frame} arg - Frame.
- _dependencies - dependencies for plugin.
- _state - inner state.
    '''
    def __init__(self, frame: Frame | None = None):
        '''Init plugin method.'''
        self.frame = frame
        self._has_frame = isinstance(frame, Frame)
        super().__init__()
        self._dependencies = []
        self._state = {'included': False}
    def work(self):
        '''Main plugin method.'''
        raise PluginIsNotWorkingError
    def include(self):
        '''Include plugin to [Frame] method.'''
        self._check_dependencies()
        if not self._state['included']:
            self.frame: Frame = Frame() if not self._has_frame else self.frame
            self.frame.Code(f'import {", ".join(self._dependencies)}') if self._dependencies is not [] else None
            self._state['included'] = True
    def _set_dependencies(self, deps: list):
        self._dependencies = deps
    def _check_dependencies(self):
        for dep in self._dependencies:
            try: importlib.import_module(dep)
            except ImportError:
                PluginError(f'Dependencies error: {dep} is not installed.')
    def _check(self):
        if not self._state['included']: raise PluginError("Use include method before using operations. \nUse 'plugin.include()'' (example) for include lib.")
    def __call__(self, *args, **kwds):
        return self.frame



class PluginRegistry:
    _plugins = {}
    
    @classmethod
    def register(cls, name, plugin_class):
        cls._plugins[name] = plugin_class
    
    @classmethod
    def get_plugin(cls, name, frame) -> type[PluginRegistry]:
        return cls._plugins[name](frame)
    
    @classmethod
    def list_plugin(cls):
        return cls._plugins

def register_plugin(name):
    def decorator(cls):
        PluginRegistry.register(name, cls)
        return cls
    return decorator




'''
# Simple Nets Core
Simplified module of net security concept for beginers.
'''
from .nets_core import (
    GrayHackApi, 
    WhiteHackApi, 
    DatsSecureApi, 
    HackModuleApi, 
    HashCryptoApi, 
    RandCryptoApi
)




import threading
import pickle
import json
from typing import Dict, Any
from .exceptions import *
from .funcs import exec_and_return, str_to_int
import ast, inspect


class _CodeGenerator:
    '''Basic system engine for code generation.'''
    def __init__(self):
        self._code = []
        self._cache = 0
    def _new_line(self, text: str): 
        self._code.append(text)
        self._cache += 1
        return self
    def _gen_temp_var(self, k: int = 0):
        var_name = f'__tmp_{self._cache - k}'
        self._cache += 1
        return var_name
    def _gen_id(self, k: int = 0):
        id = f'_id{len(self._code)+self._cache+k}{self._cache * (k+1)}{k}'
        self._cache += 1
        return id
    def _comentary(self, *args):
        self._new_line(f'"""{"\n".join(args)}"""')


class Framer(_CodeGenerator):
    '''
# Framer

Main context manager class: Framer.

For system.
    '''
    def __init__(self):
        super().__init__()
        self._vars = {}
        self._aliases = {}
        self._types = {}
        self._lock = threading.RLock()
        self._new_code_line = self._new_line
    
    def var(self, name:str, value):
        self._vars[name] = value
        return self

    def op(self, a, b, 
           operator: str = '+'):
        result_var = self._gen_temp_var()
        res = f'{a} {operator} {b}'
        code_line = f'{result_var} = {res}'
        self._code.append(code_line)
        return result_var
    
    def execute(self, algoritm = 1):
        try:
            final_code = "\n".join(self._code)
            local_scope = self._vars.copy()
            compiled = compile(final_code, '<string>', 'exec')
            exec(compiled, {}, local_scope)
            if algoritm == 1:
                return_var = "__frame_return_value"
                return local_scope.get(return_var) 
            return local_scope
        except Exception as e:
            raise FrameExecutionError(f"Error in frame execution: {e}\nCode:\n{final_code}")
    
    def get_thread_safe(self, name):
        with self._lock: return self._vars.get(self._aliases[name])

    def __enter__(self):
        System.last_framer = System.framer
        System.framer = self
        return self
    def __exit__(self, *args, **kwargs): pass




class SystemClass:
    '''
# System

System context class: System.

Has main [{framer}: Framer], [{last_framer}: Framer] and [{framers}: dict] self parameters.

For system.'''
    def __init__(self):
        self.framer = Framer()
        self.last_framer = self.framer
        self.framers = {'basic': Framer(), 'temp': Framer()}
    
    @classmethod
    def get_global(cls):
        if not hasattr(cls, '_global'):
            cls._global = cls()
        return cls._global

System = SystemClass().get_global()

class SystemOP:
    '''
    # System operations
    Basic simple api for [System] and [Framer].
    '''
    def match(condition: str, 
              true_block: str, 
              false_block: str = None, 
              framer: Framer | None = None):
        'Simple condition blok for frame.'
        framer = System.framer if framer == None else framer
        framer._comentary('condition block', condition)
        cache = framer._gen_id(len(framer._aliases) + len(framer._code))
        Var(f'__condition_temp{cache}', condition, with_eval=True, framer=framer)
        framer._new_code_line(f'if __condition_temp{cache}:')
        new_true_block = ''
        for i in true_block.split('\n'):
            if i.strip() != '': new_true_block += '\n    ' + i
        framer._new_code_line(f'    {new_true_block}')
        if false_block:
            framer._new_code_line('else:')
            new_false_block = ''
            for i in false_block.split('\n'):
                if i.strip() != '': new_false_block += '\n    ' + i
            framer._new_code_line(f'    {new_false_block}')
    def to_last():
        s = System.framer
        System.framer = System.last_framer
        System.last_framer = s


class Var:
    '''
# Variable

Abstraction api class for [Framer] and [System].

### Args:

- {name}: str - name of variable.
- {value}: Any - value of variable.
- {type}: str - type hint for debug in code.
- {to_repr}: bool - if true, value in variable will be repr(value).
- {with_eval}: bool - if true, value in variable will be ```f'eval({repr(value)})'```.
- {framer}: Framer | None - Framer object.
- {check_for_constant}: bool - check is variable constant if true


### Example: 
```
ctx = Framer() # creating context
Var('x', 10, framer = ctx) # setting variable
```
'''
    def __init__(self, 
                 name: str, 
                 value, 
                 type: str = 'int', 
                 to_repr: bool = True, 
                 with_eval: bool = False,
                 framer: Framer | None | str = 'System',
                 check_for_constant: bool = True):
        framer: Framer = Framer() if framer == None else System.framer if isinstance(framer, str) and framer.lower().strip() == 'system' else framer
        param_name = framer._gen_temp_var()
        self.name = param_name
        self.value = value
        to_repr = True if with_eval else to_repr
        val = repr(value) if to_repr else value
        val = f'eval({val})' if with_eval else val
        with framer._lock:
            if name in framer._types and check_for_constant:
                if framer._types[name] == 'const':
                    raise VariableTypeError(f'Variable already declared with type [{type}]. It cannot be overwritten.')
            framer.var(param_name, value)
            framer._types[name] = type
            type = type.replace("const", "int")
            framer._new_line(f'{name}: {type} = {val}')
            framer._aliases[name] = param_name
        self.framer = framer
def Get(name: str, 
        framer: Framer | None | str = 'System'):
    '''Get variable by {name} from {framer} method.'''
    framer: Framer = Framer() if framer == None else System.framer if isinstance(framer, str) and framer.lower().strip() == 'system' else framer
    return framer.get_thread_safe(name)

class Return:
    '''
# Return
 
Method to set variable to return with Exec() method.

### Args: 
- {value}: Var - Variable for return (object).
- {framer}: Framer | None - Framer object.

### Example: 
Code:
```
with Frame() as f: # creating context
    # setting variables
    x = Var('x', 10)
    y = Var('y', 50)
    res = Var('test', Get('x') * Get('y')) 
    Return(res) # setting variable to return
print('result:', Exec()) # executing code
```
Output:
```
result: 500
```'''
    def __init__(self, 
                 value: Var, 
                 framer: Framer | None | str = 'System'):
        framer: Framer = Framer() if framer == None else System.framer if isinstance(framer, str) and framer.lower().strip() == 'system' else framer
        return_var = "__frame_return_value"
        try: framer.var(return_var, f'{framer._vars.get(value.name)}')
        except AttributeError:
            raise FramerError(f'Exception in atribute parsing. \nObject [{value}, {type(value)}] has no atribute .name to create return. \nPlease, use [value] declaration like [`res = Var(...); Return(res, ...)`].')
        self.framer = framer
class Code:
    '''
# Code append

Method to append code in Framer.

### Args:
- {code}: str - code for paste to framer.
- {framer}: Framer | None - framer object.

### Example:
Code:
```
with Frame() as f:
    x = Var('x', 10)
    y = Var('y', 50)
    Code('result = x + y')
    Var('test', Get('x') * Get('y')) 
    Var('res', 'test + result', with_eval=True)
print('result:', exec_and_return(f.compile(), 'res'))
```
Output:
```
result: 560
```'''
    def __init__(self, 
                 code: str, 
                 framer: Framer | None | str = 'System'):
        framer: Framer = Framer() if framer == None else System.framer if isinstance(framer, str) and framer.lower().strip() == 'system' else framer
        framer._new_code_line(code)
        self._code = code
        self.framer = framer

def Exec(framer: Framer | None | str = 'System', algoritm: int = 1):
    '''
Execution of [Frame] method.
    '''
    framer: Framer = Framer() if framer == None else System.framer if isinstance(framer, str) and framer.lower().strip() == 'system' else framer
    with framer._lock:
        return framer.execute(algoritm)
    
class Frame:
    '''
# Frame
Abstraction api for all [Framer] and [System] methods.

(framer in functions is Frame.framer)

You can use with to create context, and call [Frame] object like `frame()` to get framer.

### Args of initialization:
- {framer}: str | Framer = 'new' - framer context object for frame.
- {safemode}: bool - if safemode true, Exec method will be is not available.
- {name}: str - framer name in [System.framers]
- {save_while_exit}: bool - if true, while will be called [__exit__], context will be automaticly saved.
- {save_args}: list - list of args [name, format] for method save.

### Example usage:
Code:
```
with Frame(safemode=False) as f:
    f.Var('x', 10)
    f.Var('y', 50)
    SystemOP.match('x > y', 'print("x bigger")', 'print("y bigger")')
    f.Var('test', Get('x') * Get('y')) 
code = f.compile()
print('result:', exec_and_return(code, 'test'))
```
Output:
```
y bigger
result: 500
```'''
    def __init__(self, 
                 framer: str | Framer = 'new', 
                 safemode: bool = True, 
                 name: str = 'f1',
                 save_while_exit: bool = False,
                 save_args: list = ['ctx', 'pickle']):
        self.__saving = [save_while_exit, save_args]
        self.framer: Framer = Framer() if framer == 'new' else framer
        self.__safemode = safemode
        self._name = name
        System.framers[name] = self.framer
    def Sys(self) -> SystemOP: 
        '''Return [System] class.'''
        return SystemOP
    def System(self) -> SystemClass: 
        '''Return [System] class.'''
        return System
    def set_System(self, new_sys: SystemClass = SystemClass()) -> SystemClass:
        global System
        System = new_sys
    def Var(self, 
            name: str, 
            value, 
            type: str = 'int', 
            to_repr: bool = True,
            with_eval: bool = False,
            check_for_constant: bool = False) -> Var:
        '''Creating variable.'''
        return Var(name = name, 
                   value = value, 
                   type = type, 
                   to_repr = to_repr, 
                   with_eval = with_eval, 
                   framer = self.framer, 
                   check_for_constant = check_for_constant)
    def Get(self, name: str) -> Any: 
        '''Get variable by name.'''
        return Get(name, self.framer)
    def Return(self, name: Var) -> Return: 
        '''Set of variable to return.'''
        return Return(name, self.framer)
    def Code(self, code: str, comentary: bool = True, *comentaries) -> Code:
        '''Append code to frame.'''
        if comentary:
            self.framer._comentary('code section', f'Framer: {self._name}', f'Safemode: {self.__safemode}', *comentaries)
        return Code(code, self.framer)
    def Exec(self, algoritm: int = 1) -> Any:
        '''Executing code of frame.'''
        if not self.__safemode: return Exec(self.framer, algoritm)
        else: raise FrameApiError('Exec is not avialable in safemode.')
    def compile(self) -> str: 
        '''Get full code of frame.'''
        return '\n'.join(self.framer._code)
    def reset(self) -> Frame: 
        '''Recreate framer.'''
        self.framer = Framer()
        return self
    def register(self, name: str = None):
        'Register any construction to frame decortor.'
        def decorator(func):
            func_name = name or func.__name__
            try:
                source_code = inspect.getsource(func)
                lines = source_code.split('\n')[1:]
                tabbed = False
                can_edit_tabs = True
                tabs = 0
                cleaned_code = []
                for i in lines:
                    if i.startswith('   '): 
                        t = 0
                        for local_i in i: 
                            if local_i == ' ':
                                t += 1
                            if t % 4 == 0: 
                                if can_edit_tabs: tabs = t/4
                            tabbed = True
                        can_edit_tabs = False
                    if not i.startswith('   '): tabbed = False; can_edit_tabs = False
                    if tabbed: 
                        tabs_c = int(4 * tabs)
                        cleaned_code.append(i[tabs_c:])
                    else: 
                        cleaned_code.append(i)
                cleaned_code = '\n'.join(cleaned_code)
                type = cleaned_code.split(" ")[0]
                self.Code(f"\n# Registred construction: {type} {func_name}\n{cleaned_code}", True, f'Registring [{type} {func_name}] construcntion.')
            except Exception as e:
                print(f"Warning: Could not register source code for {func_name}: \n{e}")
                self.Var(func_name, func, to_repr=False)
            return func
        return decorator
    def save(self, filename: str, format: str = 'pickle') -> Frame:
        '''
        ## Saving frame to file.
        ### Args:
        - {filename}: str - file name
        - {format}: str - saving format ('pickle' or 'json')
        '''
        data = self.data
        try:
            if format == 'pickle':
                with open(filename, 'wb') as f: pickle.dump(data, f)
            elif format == 'json':
                json_data = self._serealize('json')
                with open(filename, 'w', encoding='utf-8') as f: 
                    json.dump(json_data, f, indent=2, ensure_ascii=False)
            else: raise FrameApiError(f"Unsupported format: {format}")
            return self
        except Exception as e:
            raise FrameApiError(f"Save failed: {e}")

    def load(self, filename: str = 'ctx', format: str = 'pickle', local_safemode: bool = True) -> Frame:
        '''
        ## Loading frame from file.
        ### Args:
        - {filename}: str - file name
        - {format}: str - loading format ('pickle' or 'json')
        '''
        try:
            if format == 'pickle':
                with open(filename, 'rb') as f: data = pickle.load(f)
            elif format == 'json':
                with open(filename, 'r', encoding='utf-8') as f: data = json.load(f)
                restored_vars = {}
                for k, v in data['framer']['_vars'].items():
                    try: restored_vars[k] = ast.literal_eval(v)
                    except: 
                        try:
                            if not self.__safemode or not local_safemode: 
                                restored_vars[k] = eval(v)
                            else: restored_vars[k] = v
                        except Exception as e: 
                            raise FrameApiError(f'Error in parsing parameters: {e}')
                data['framer']['_vars'] = restored_vars
            else: raise FrameApiError(f"Unsupported format: {format}")
            self._load_data(data)
            return self
        except Exception as e:
            raise FrameApiError(f"Load failed: {e}")
    @property
    def data(self):
        return {
                'framer': {
                    '_code': self.framer._code,
                    '_vars': self.framer._vars,
                    '_aliases': self.framer._aliases,
                    '_types': self.framer._types
                },
                'saving': self.__saving,
                'safemode': self.__safemode,
                'name': self._name
                }

    def _serealize(self, form: str = 'pickle' ):
        data = self.data
        json_data = {
            'framer': {
                '_code': data['framer']['_code'],
                '_vars': {k: str(v) for k, v in data['framer']['_vars'].items()},  # Приводим к строке для JSON
                '_aliases': data['framer']['_aliases'],
                '_types': data['framer']['_types']
            },
            'safemode': data['safemode'],
            'saving': data['saving'],
            'name': data['name']
        }
        return data if form == 'pickle' else json_data
    def _load_data(self, data: dict):
        self.framer._code = data['framer']['_code']
        self.framer._vars = data['framer']['_vars'] 
        self.framer._aliases = data['framer']['_aliases']
        self.framer._types = data['framer']['_types']
        self.__safemode = data['safemode']
        self.__saving = data['saving']
        self._name = data['name']
        return self
    def _get_safemode(self): return self.__safemode
    def __enter__(self): 
        self.framer.__enter__()
        return self
    def __exit__(self, *args, **kwargs): 
        if self.__saving[0]: self.save(*self.__saving[1])
    def __call__(self, *args, **kwds):
        return self.framer
    def __getitem__(self, index):
        return self.Get(index)





Var('name', 'frame', framer=System.framers['basic'])

if __name__ == '__main__':
    with Frame() as f:
        x = Var('x', 10)
        y = Var('y', 50)
        SystemOP.match('x > y', 'print("x bigger")', 'print("y bigger")')
        res = Var('test', Get('x') * Get('y')) 
        Return(res)  
    print(Exec())  # → 500
    with Frame() as f:
        x = Var('x', 10)
        y = Var('y', 50)
        res = Var('res', 'x + y', with_eval=True)
        Return(res)
    print(Exec())
    code = f.compile()
    print(exec_and_return(code, 'res')) # 60
    with Frame() as f:
        x = Var('x', 10)
        y = Var('y', 50)
        Code('result = x + y')
        Var('test', Get('x') * Get('y')) 
        Var('res', 'test + result', with_eval=True)
    print(exec_and_return(f.compile(), 'res')) # 560
    with Frame(save_while_exit=True, save_args=['ctx.json', 'json']) as f:
        f.Var('x', 10)
        f.Var('y', 50)
        SystemOP.match('x > y', 'print("x bigger")', 'print("y bigger")')
        f.Var('test', Get('x') * Get('y')) 
        print(f['test'], '========')
        @f.register()
        def test(): 
            print('testing')
        @f.register()
        class Test():
            hello = 'World'
            pass
    with Frame().load('ctx.json', format='json') as f:
        code = f.compile()
        print('result:', exec_and_return(code, 'test'))
    print('\n\n======= CODE =======')
    print(code)
    '''
y bigger
500
x + y
60
560
500 ========
y bigger
result: <function test at 0x10622da60>
test1
f1
<__main__.Frame object at 0x1061fc380>, test, sgc, 6585, False
6585


======= CODE =======
x: int = 10
y: int = 50
"""condition block
x > y"""
__condition_temp_id13305: int = eval('x > y')
if __condition_temp_id13305:
    
    print("x bigger")
else:
    
    print("y bigger")
test: int = 500
"""code section
Framer: f1
Safemode: True
Registring [def test] construcntion."""

# Registred construction: def test
def test(): 
    print('testing')

"""code section
Framer: f1
Safemode: True
Registring [class Test] construcntion."""

# Registred construction: class Test
class Test():
    hello = 'World'
    pass
'''


import sys
import os
import subprocess
from .frame_core import *
from .frame_core import _framecore_version_


def help():
    help_text = '''
=== FrameFile Starter HELP ===
Args -

    GROUP start:
        stjson [name] - start json framefile
        stpick [name] - start pickle framefile
        GROUP_ARGS:
            -s - turn safemode
            NOTE: without safemode file can show errors
        GROUP_FORMAT:
            command [name] -[safemode]

    GROUP help:
        getcode_s / getcode [name] - get code of frame (safe/unsafe)
        compile_s / compile [name] [out_name] - compile framefile to .py file (safe/unsafe)
        is_safe_s / is_safe [name] - check safemode for framefile (safe/unsafe)
        NOTE: without safemode file can show errors
        NOTE: safe/unsafe - load mode
        GROUP_ARGS:
            -json - json format
            -pick - pickle format
        GROUP_FORMAT:
            command [args] -[format]
    
    GROUP other:
        -v - show frame version
        -h - show this

==============================
'''
    print(help_text)

def getframe(filename, format, safe):
    return Frame(safemode=safe).load(filename, format, safe)

def getcode(filename, format, safe):
    frame = getframe(filename, format, safe)
    return frame.compile()

def print_code(args: list, ln: int, safe: bool = False):
    if ln < 1:
        raise KeyError("Filename is required")
    filename = args[0]
    format_type = 'pickle' if ln > 1 and args[1] == '-pick' else 'json'
    print(getcode(filename, format_type, safe))

def compile_code(args: list, ln: int, safe: bool = False):
    if ln < 1:
        raise KeyError("Filename is required")
    filename = args[0]
    output_name = args[1] if ln > 1 else f'bin_framefile_{filename}.py'
    format_type = 'pickle' if ln > 2 and args[2] == '-pick' else 'json'
    code = getcode(filename, format_type, safe)
    with open(output_name, 'w', encoding='utf-8') as f:
        f.write(code)
    print(f"Compiled to: {output_name}")

def start(args: list, ln: int, file_type: str = 'json'):
    if ln < 1:
        raise KeyError("Filename is required")
    filename = args[0]
    safe_mode = ln > 1 and args[1] == '-s'
    output_name = f'_tmp_bin_framefile_{filename}.py'
    compile_code([filename, output_name, f'-{file_type}'], 3, safe_mode)
    try:
        subprocess.run([sys.executable, output_name], check=True)
    except subprocess.CalledProcessError as e:
        print(f"Error running compiled file: \n{e}")
    finally:
        if os.path.exists(output_name):
            os.remove(output_name)
        

def is_safe(args: list, ln: int, safe: bool):
    if ln < 1:
        raise KeyError("Filename is required")
    filename = args[0]
    format_type = 'pickle' if ln > 1 and args[1] == '-pick' else 'json'
    frame = getframe(filename, format_type, safe)
    return frame._get_safemode()




def main():
    if len(sys.argv) < 2:
        print('Unknown Command. Type -h for help.')
        return
    command = sys.argv[1]
    args = sys.argv[2:]
    args_count = len(args)
    command_handlers = {
        'stjson': lambda: start(args, args_count, 'json') if args_count >= 1 else FileNotFoundError("Filename required"),
        'stpick': lambda: start(args, args_count, 'pick') if args_count >= 1 else FileNotFoundError("Filename required"),
        'getcode': lambda: print_code(args, args_count, False) if args_count >= 1 else FileNotFoundError("Filename required"),
        'getcode_s': lambda: print_code(args, args_count, True) if args_count >= 1 else FileNotFoundError("Filename required"),
        'compile': lambda: compile_code(args, args_count, False) if args_count >= 1 else FileNotFoundError("Filename required"),
        'compile_s': lambda: compile_code(args, args_count, True) if args_count >= 1 else FileNotFoundError("Filename required"),
        'is_safe': lambda: print('Safemode:', is_safe(args, args_count, False)) if args_count >= 1 else FileNotFoundError("Filename required"),
        'is_safe_s': lambda: print('Safemode:', is_safe(args, args_count, True)) if args_count >= 1 else FileNotFoundError("Filename required"),
        '-h': lambda: help(),
        '-v': lambda: print(f'Frames Concept v{_framecore_version_}'),
    }
    command = command.lower()
    if command in command_handlers:
        try:
            command_handlers[command]()
        except Exception as e:
            print(f"Error: {e}")
            sys.exit(1)
    else:
        print('Unknown Command. Type -h for help.')

if __name__ == '__main__':
    main()


import unittest
import sys
import os

sys.path.append(os.path.dirname(os.path.dirname(__file__)))

from frame import Frame, FastGet
from frame import MathPlugin

class TestMathPlugin(unittest.TestCase):
    
    def test_plugin_initialization(self):
        """Тест инициализации плагина"""
        frame = Frame(safemode=False)
        plugin = MathPlugin(frame)
        
        self.assertEqual(plugin.work(), 'mathplugin <v0.1.1>')
        self.assertFalse(plugin._state['included'])
    
    def test_plugin_inclusion(self):
        """Тест включения плагина"""
        frame = Frame(safemode=False)
        plugin = MathPlugin(frame)
        plugin.include()
        
        self.assertTrue(plugin._state['included'])
        # Проверяем, что зависимости добавлены в код
        code = frame.compile()
        self.assertIn('import math, cmath', code)
    
    def test_math_operations(self):
        """Тест математических операций"""
        frame = Frame(safemode=False)
        plugin = MathPlugin(frame).include()
        
        # Тест параболы
        plugin.parabola(5, 'result')
        frame.Exec()
        self.assertEqual(FastGet(frame, 'result'), 25)
        
        # Тест квадратного корня
        plugin.sqrt(16, 'sqrt_result')
        frame.Exec()
        self.assertEqual(FastGet(frame, 'sqrt_result'), 4.0)
    
    def test_complex_math(self):
        """Тест сложных математических функций"""
        frame = Frame(safemode=False)
        plugin = MathPlugin(frame).include()
        
        # Тест дискриминанта
        plugin.discriminant(1, -3, 2, 'disc_result')
        frame.Exec()
        result = FastGet(frame, 'disc_result')
        self.assertEqual(result[2], 1)  # Дискриминант должен быть 1
    
    def test_sigmoid_function(self):
        """Тест функции сигмоиды"""
        frame = Frame(safemode=False)
        plugin = MathPlugin(frame).include()
        
        plugin.sigmoid(0, 'sigmoid_result')
        frame.Exec()
        result = FastGet(frame, 'sigmoid_result')
        self.assertAlmostEqual(result, 0.5, places=5)

if __name__ == '__main__':
    unittest.main()